class TokenAttributeBuilderFunctionsQueryBuilder extends GeneralFunctionsQueryBuilder {
  constructor(elements) {
    super(elements);
    this.elements = elements;
  }

  //#region General functions of the Token Builder
  tokenTypeSelector() {
    this.hideEverything();
    switch (this.elements.positionalAttr.value) {
      case 'word':
        this.wordBuilder();
        break;
      case 'lemma':
        this.lemmaBuilder();
        break;
      case 'english-pos':
        this.englishPosHandler();
        break;
      case 'german-pos':
        this.germanPosHandler();
        break;    
      case 'simple-pos-button':
        this.simplePosBuilder();
        break;
      case 'empty-token':
        this.emptyTokenHandler();
        break;
      default:
        this.wordBuilder();
        break;
    }
  }

  tokenChipFactory(prettyQueryText, tokenText) {
    tokenText = encodeURI(tokenText);
    let builderElement;
    let queryChipElement;
    builderElement = document.createElement('div');
    builderElement.innerHTML = `
    <div class='chip col s2 l2' style='margin-top:20px;' data-tokentext='${tokenText}'>
      ${prettyQueryText}
      <i class='material-icons close'>close</i>
    </div>`;
    queryChipElement = builderElement.firstElementChild;
    queryChipElement.addEventListener('click', () => {this.deleteTokenAttr(queryChipElement);});
    this.elements.tokenQuery.appendChild(queryChipElement);
  }

  deleteTokenAttr(attr) {
    if (this.elements.tokenQuery.childNodes.length < 2) {
      this.elements.tokenQuery.removeChild(attr);
      this.wordBuilder();
    } else {
      this.elements.tokenQuery.removeChild(attr);
    }

  }

  addTokenToQuery() {
    let c;
    let tokenQueryContent = ''; //for ButtonFactory(prettyQueryText)
    let tokenQueryText = ''; //for ButtonFactory(queryText)
    this.elements.cancelBool = false;
    let tokenIsEmpty = false;

    if (this.elements.ignoreCase.checked) {
      c = ' %c';
    } else {
      c = '';
    }

    for (let element of this.elements.tokenQuery.childNodes) {
      tokenQueryContent += ' ' + element.firstChild.data + ' ';
      tokenQueryText += decodeURI(element.dataset.tokentext);
      if (element.innerText.indexOf('empty token') !== -1) {
        tokenIsEmpty = true;
      }
    }

    if (this.elements.tokenQueryFilled === false) {
      switch (this.elements.positionalAttr.value) {
        case 'word':
          if (this.elements.wordInput.value === '') {
            this.disableTokenSubmit();
          } else {
            tokenQueryContent += `word=${this.elements.wordInput.value}${c}`;
            tokenQueryText += `word="${this.elements.wordInput.value}"${c}`;
            this.elements.wordInput.value = '';
          }
          break;
        case 'lemma':
          if (this.elements.lemmaInput.value === '') {
            this.disableTokenSubmit();
          } else {
            tokenQueryContent += `lemma=${this.elements.lemmaInput.value}${c}`;
            tokenQueryText += `lemma="${this.elements.lemmaInput.value}"${c}`;
            this.elements.lemmaInput.value = '';
          }
          break;
        case 'english-pos':
          if (this.elements.englishPos.value === 'default') {
            this.disableTokenSubmit();
          } else {
            tokenQueryContent += `pos=${this.elements.englishPos.value}`;
            tokenQueryText += `pos="${this.elements.englishPos.value}"`;
            this.elements.englishPos.value = '';
          }
          break;
        case 'german-pos':
          if (this.elements.germanPos.value === 'default') {
            this.disableTokenSubmit();
          } else {
            tokenQueryContent += `pos=${this.elements.germanPos.value}`;
            tokenQueryText += `pos="${this.elements.germanPos.value}"`;
            this.elements.germanPos.value = '';
          }
          break;
        case 'simple-pos-button':
          if (this.elements.simplePos.value === 'default') {
            this.disableTokenSubmit();
          } else {
            tokenQueryContent += `simple_pos=${this.elements.simplePos.value}`;
            tokenQueryText += `simple_pos="${this.elements.simplePos.value}"`;
            this.elements.simplePos.value = '';
          }
          break;
        default:
          this.wordBuilder();
          break;
      }
    }

    // cancelBool looks in disableTokenSubmit() whether a value is passed. If the input fields/dropdowns are empty (cancelBool === true), no token is added.
    if (this.elements.cancelBool === false) {
      // Square brackets are added only if it is not an empty token (where they are already present).
      if (tokenIsEmpty === false) {
        tokenQueryText = '[' + tokenQueryText + ']';
        }
      this.queryChipFactory('token', tokenQueryContent, tokenQueryText);
      this.hideEverything();
      this.elements.positionalAttrArea.classList.add('hide');
      this.elements.tokenQuery.innerHTML = '';
    }

  }

  disableTokenSubmit() {
    this.elements.cancelBool = true;
    this.elements.tokenSubmitButton.classList.add('red');
    this.elements.noValueMessage.classList.remove('hide');
    setTimeout(() => {
      this.elements.tokenSubmitButton.classList.remove('red');
    }, 500);
    setTimeout(() => {
      this.elements.noValueMessage.classList.add('hide');
    }, 3000);
  }

  inputFieldHandler() {
    let input;

    if (this.elements.wordBuilder.classList.contains('hide') === false) {
      input = this.elements.wordInput;
    } else {
      input = this.elements.lemmaInput;
    }

    if (input.value === '') {
      this.elements.incidenceModifiersButton.firstElementChild.classList.add('disabled');
      this.elements.or.classList.add('disabled');
      this.elements.and.classList.add('disabled');
    } else {
      this.elements.incidenceModifiersButton.firstElementChild.classList.remove('disabled');
      this.elements.or.classList.remove('disabled');
      this.elements.and.classList.remove('disabled');
    }
  }

    //#endregion General functions of the Token Builder

    //#region Dropdown Select Handler

  lemmaBuilder() {
    this.hideEverything();
    this.elements.lemmaInput.value = '';
    this.elements.lemmaBuilder.classList.remove('hide');
    this.elements.inputOptions.classList.remove('hide');
    this.elements.incidenceModifiersButton.classList.remove('hide');
    this.elements.incidenceModifiersButton.firstElementChild.classList.add('disabled');
    this.elements.conditionContainer.classList.remove('hide');
    this.elements.ignoreCaseCheckbox.classList.remove('hide');

    this.elements.incidenceModifiersButton.firstElementChild.classList.add('disabled');
    this.elements.or.classList.add('disabled');
    this.elements.and.classList.add('disabled');
  }

  englishPosHandler() {
    this.hideEverything();
    this.elements.englishPosBuilder.classList.remove('hide');
    this.elements.incidenceModifiersButton.classList.remove('hide');
    this.elements.conditionContainer.classList.remove('hide');
    this.elements.incidenceModifiersButton.firstElementChild.classList.remove('disabled');
    this.elements.or.classList.remove('disabled');
    this.elements.and.classList.remove('disabled');
    
    // Resets materialize select dropdown
    let selectInstance = M.FormSelect.getInstance(this.elements.englishPos);
    selectInstance.input.value = 'English pos tagset';
    this.elements.englishPos.value = 'default';
  }
  
  germanPosHandler() {
    this.hideEverything();
    this.elements.germanPosBuilder.classList.remove('hide');
    this.elements.incidenceModifiersButton.classList.remove('hide');
    this.elements.conditionContainer.classList.remove('hide');
    this.elements.incidenceModifiersButton.firstElementChild.classList.remove('disabled');
    this.elements.or.classList.remove('disabled');
    this.elements.and.classList.remove('disabled');

    // Resets materialize select dropdown
    let selectInstance = M.FormSelect.getInstance(this.elements.germanPos);
    selectInstance.input.value = 'German pos tagset';
    this.elements.germanPos.value = 'default';
  }

  simplePosBuilder() {
  this.hideEverything();
  this.elements.simplePosBuilder.classList.remove('hide');
  this.elements.incidenceModifiersButton.classList.remove('hide');
  this.elements.conditionContainer.classList.remove('hide');
  this.elements.simplePos.selectedIndex = 0;
  this.elements.incidenceModifiersButton.firstElementChild.classList.remove('disabled');
  this.elements.or.classList.remove('disabled');
  this.elements.and.classList.remove('disabled');
  
  // Resets materialize select dropdown
  let selectInstance = M.FormSelect.getInstance(this.elements.simplePos);
  selectInstance.input.value = 'simple_pos tagset';
  this.elements.simplePos.value = 'default';
  M.FormSelect.init(
    selectInstance,
    {
      dropdownOptions: {
        direction: 'bottom',
        coverTrigger: false
      }
    }
  )

  }

  emptyTokenHandler() {
    this.tokenChipFactory('empty token', '[]');
    this.elements.tokenQueryFilled = true;
    this.hideEverything();
    this.elements.incidenceModifiersButton.classList.remove('hide');
    this.elements.incidenceModifiersButton.firstElementChild.classList.remove('disabled');

  }
  //#endregion Dropdown Select Handler
  
    //#region Options to edit your token - Wildcard Charakter, Option Group, Incidence Modifiers, Ignore Case, 'and', 'or'

  inputOptionHandler(elem) {
    let input;

    if (this.elements.wordBuilder.classList.contains('hide') === false) {
      input = this.elements.wordInput;
    } else {
      input = this.elements.lemmaInput;
    }

    if (elem === this.elements.optionGroup) {
      input.value  += '(option1|option2)';
      let firstIndex = input.value.indexOf('option1');
      let lastIndex = firstIndex + 'option1'.length;
      input.focus();
      input.setSelectionRange(firstIndex, lastIndex);
    } else if (elem === this.elements.wildcardChar) {
      input.value  += '.';
    }
    this.inputFieldHandler();
  }

  nSubmitHandler() {
    let instance = M.Modal.getInstance(this.elements.exactlyN);
    instance.close();
    
    switch (this.elements.positionalAttr.value) {
      case 'word':
        this.elements.wordInput.value += ' {' + this.elements.nInput.value + '}';
        break;
      case 'lemma':
        this.elements.lemmaInput.value += ' {' + this.elements.nInput.value + '}';
        break;
      case 'english-pos':
        this.elements.tokenQueryFilled = true;
        this.tokenChipFactory(`pos=${this.elements.englishPos.value}`, `pos="${this.elements.englishPos.value}"`);
        this.tokenChipFactory('{' + this.elements.nInput.value + '}', '{' + this.elements.nInput.value + '}');
        this.elements.englishPosBuilder.classList.add('hide');
        this.elements.incidenceModifiersButton.classList.add('hide');
        break;
      case 'german-pos':
        this.elements.tokenQueryFilled = true;
        this.tokenChipFactory(`pos=${this.elements.germanPos.value}`, `pos="${this.elements.germanPos.value}"`);
        this.tokenChipFactory('{' + this.elements.nInput.value + '}', '{' + this.elements.nInput.value + '}');
        this.elements.germanPosBuilder.classList.add('hide');
        this.elements.incidenceModifiersButton.classList.add('hide');
        break;
      case 'simple-pos-button':
        this.elements.tokenQueryFilled = true;
        this.tokenChipFactory(`simple_pos=${this.elements.simplePos.value}`, `simple_pos="${this.elements.simplePos.value}"`);
        this.tokenChipFactory('{' + this.elements.nInput.value + '}', '{' + this.elements.nInput.value + '}');
        this.elements.simplePosBuilder.classList.add('hide');
        this.elements.incidenceModifiersButton.classList.add('hide');
        break;
      case 'empty-token':
        this.tokenChipFactory('{' + this.elements.nInput.value + '}', '{' + this.elements.nInput.value + '}');
        break;
      default:
        break;
    }

  }

  nmSubmitHandler() {
    let instance = M.Modal.getInstance(this.elements.betweenNM);
    instance.close();

    switch (this.elements.positionalAttr.value) {
      case 'word':
        this.elements.wordInput.value += `{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`;
        break;
      case 'lemma':
        this.elements.lemmaInput.value += `{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`;
        break;
      case 'english-pos':
        this.elements.tokenQueryFilled = true;
        this.tokenChipFactory(`pos=${this.elements.englishPos.value}`, `pos="${this.elements.englishPos.value}"`);
        this.tokenChipFactory(`{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`, `{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`);
        this.elements.englishPosBuilder.classList.add('hide');
        this.elements.incidenceModifiersButton.classList.add('hide');
        break;
      case 'german-pos':
        this.elements.tokenQueryFilled = true;
        this.tokenChipFactory(`pos=${this.elements.germanPos.value}`, `pos="${this.elements.germanPos.value}"`);
        this.tokenChipFactory(`{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`, `{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`);
        this.elements.germanPosBuilder.classList.add('hide');
        this.elements.incidenceModifiersButton.classList.add('hide');
        break;
      case 'simple-pos-button':
        this.elements.tokenQueryFilled = true;
        this.tokenChipFactory(`simple_pos=${this.elements.simplePos.value}`, `simple_pos="${this.elements.simplePos.value}"`);
        this.tokenChipFactory(`{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`, `{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`);
        this.elements.simplePosBuilder.classList.add('hide');
        this.elements.incidenceModifiersButton.classList.add('hide');
        break;
      case 'empty-token':
        this.tokenChipFactory(`{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`, `{${this.elements.nmInput.value}, ${this.elements.mInput.value}}`);
        break;
      default:
        break;
    }
  }

  incidenceModifiersHandler(elem) {
    // For word and lemma, the incidence modifiers are inserted in the input field. For the others, one or two chips are created which contain the respective value of the token and the incidence modifier.
    if (this.elements.positionalAttr.value === 'empty-token') {
      this.tokenChipFactory(elem.innerText, elem.dataset.token);
    } else if (this.elements.positionalAttr.value === 'english-pos') {
      this.tokenChipFactory(`pos=${this.elements.englishPos.value}`, `pos="${this.elements.englishPos.value}"`);
      this.tokenChipFactory(elem.innerText, elem.dataset.token);
      this.elements.englishPosBuilder.classList.add('hide');
      this.elements.incidenceModifiersButton.classList.add('hide');
      this.elements.tokenQueryFilled = true;
    } else if (this.elements.positionalAttr.value === 'german-pos') {
      this.tokenChipFactory(`pos=${this.elements.germanPos.value}`, `pos="${this.elements.germanPos.value}"`);
      this.tokenChipFactory(elem.innerText, elem.dataset.token);
      this.elements.germanPosBuilder.classList.add('hide');
      this.elements.incidenceModifiersButton.classList.add('hide');
      this.elements.tokenQueryFilled = true;
    } else if (this.elements.positionalAttr.value === 'simple-pos-button') {
      this.tokenChipFactory(`simple_pos=${this.elements.simplePos.value}`, `simple_pos="${this.elements.simplePos.value}"`);
      this.tokenChipFactory(elem.innerText, elem.dataset.token);
      this.elements.simplePosBuilder.classList.add('hide');
      this.elements.incidenceModifiersButton.classList.add('hide');
      this.elements.tokenQueryFilled = true;
    } else {
      let input;

      if (this.elements.wordBuilder.classList.contains('hide') === false) {
        input = this.elements.wordInput;
      } else {
        input =  this.elements.lemmaInput;
      }
      input.value += elem.dataset.token;
    }

  }

  orHandler() {
    this.conditionHandler('or', ' | ');
  }

  andHandler() {
    this.conditionHandler('and', ' & ');
  }

  conditionHandler(conditionText, conditionQueryContent) {
    this.hideEverything();
    let tokenQueryContent;
    let tokenQueryText;
    let c;

    if (this.elements.ignoreCase.checked) {
      c = ' %c';
    } else {
      c = '';
    }

    switch (this.elements.positionalAttr.value) {
      case 'word':
        tokenQueryContent = `word=${this.elements.wordInput.value}${c}`;
        tokenQueryText = `word="${this.elements.wordInput.value}"${c}`;
        this.elements.wordInput.value = '';
        break;
      case 'lemma':
        tokenQueryContent = `lemma=${this.elements.lemmaInput.value}${c}`;
        tokenQueryText = `lemma="${this.elements.lemmaInput.value}"${c}`;
        this.elements.lemmaInput.value = '';
        break;
      case 'english-pos':
        tokenQueryContent = `pos=${this.elements.englishPos.value}`;
        tokenQueryText = `pos="${this.elements.englishPos.value}"`;
        this.elements.englishPos.value = '';
        break;
      case 'german-pos':
        tokenQueryContent = `pos=${this.elements.germanPos.value}`;
        tokenQueryText = `pos="${this.elements.germanPos.value}"`;
        this.elements.germanPos.value = '';
        break;
      case 'simple-pos-button':
        tokenQueryContent = `simple_pos=${this.elements.simplePos.value}`;
        tokenQueryText = `simple_pos="${this.elements.simplePos.value}"`;
        this.elements.simplePos.value = '';
        break;
      default:
        this.wordBuilder();
        break;
    }

    this.tokenChipFactory(tokenQueryContent, tokenQueryText);
    this.tokenChipFactory(conditionText, conditionQueryContent);
    this.wordBuilder();
  }

  //#endregion Options to edit your token - Wildcard Charakter, Option Group, Incidence Modifiers, Ignore Case, 'and', 'or'
}
